"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var typescript_1 = __importDefault(require("typescript"));
var lodash_1 = __importDefault(require("lodash"));
function transformer(program) {
    var checker = program.getTypeChecker();
    return function (context) {
        var visit = function (node) {
            node = typescript_1.default.visitEachChild(node, visit, context);
            /*
              Convert type references to type literals
        
                interface IUser {
                  username: string
                }
                type User = IUser <--- IUser is a type reference
                interface Context {
                  user: User <--- User is a type reference
                }
        
              In both cases we want to convert the type reference to
              it's primitive literals. We want:
        
                interface IUser {
                  username: string
                }
                type User = {
                  username: string
                }
                interface Context {
                  user: {
                    username: string
                  }
                }
            */
            if (typescript_1.default.isTypeReferenceNode(node)) {
                var symbol = checker.getSymbolAtLocation(node.typeName);
                var type = checker.getDeclaredTypeOfSymbol(symbol);
                var declarations = lodash_1.default.flatMap(checker.getPropertiesOfType(type), function (property) {
                    /*
                      Type references declarations may themselves have type references, so we need
                      to resolve those literals as well
                    */
                    return lodash_1.default.map(property.declarations, visit);
                });
                return typescript_1.default.createTypeLiteralNode(declarations.filter(typescript_1.default.isTypeElement));
            }
            /*
              Convert type alias to interface declaration
                interface IUser {
                  username: string
                }
                type User = IUser
            
              We want to remove all type aliases
                interface IUser {
                  username: string
                }
        
                interface User {
                  username: string  <-- Also need to resolve IUser
                }
            
            */
            if (typescript_1.default.isTypeAliasDeclaration(node)) {
                var symbol = checker.getSymbolAtLocation(node.name);
                var type = checker.getDeclaredTypeOfSymbol(symbol);
                var declarations = lodash_1.default.flatMap(checker.getPropertiesOfType(type), function (property) {
                    // Resolve type alias to it's literals
                    return lodash_1.default.map(property.declarations, visit);
                });
                // Create interface with fully resolved types
                return typescript_1.default.createInterfaceDeclaration([], [typescript_1.default.createToken(typescript_1.default.SyntaxKind.ExportKeyword)], node.name.getText(), [], [], declarations.filter(typescript_1.default.isTypeElement));
            }
            // Remove all export declarations
            if (typescript_1.default.isImportDeclaration(node)) {
                return null;
            }
            return node;
        };
        return function (node) { return typescript_1.default.visitNode(node, visit); };
    };
}
exports.transformer = transformer;
